//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "elobal.h"

#if defined (_WINDOWS)
	#include <direct.h>
	#include <windows.h>
#elif defined (__unix__)
	#include <sys/stat.h>
#endif
#include <string>
#include <iostream>
#include <time.h>

// qtils 
#include <Config.h>
#include <logger/Log.h>
#include <SystemUtils.h>

// bass
#include <Bass.h>

#include "tasks/DiverseSearchTH.h"

#include "EmcMain.h"

const string EmcMain::emcVersion = "release 1";

EmcMain::EmcMain(Config *config) {
	mConfig = config;
	mTaskClass = mConfig->Read<string>("taskclass");

	Bass::ParseGlobalSettings(mConfig);
}
EmcMain::~EmcMain() {
	
}

/* ------------------------- Main execution -------------------------- */

void EmcMain::Execute() {
	// Init TaskHandler
	TaskHandler *th = NULL;

	if(mTaskClass.compare("diverse") == 0)		th = new DiverseSearchTH(mConfig);
	else THROW("No TaskHandler available for `" + mTaskClass + "`");

	// Init command in AppLog
	Logger::CommandInit(mTaskClass, mConfig->Read<string>("command"));

	// Set working directory, based on TaskHandler
#if defined (_MSC_VER)
	Bass::SetWorkingDir(Bass::GetExpDir() + th->BuildWorkingDir());
#elif defined (__GNUC__)
	string wd = Bass::GetExpDir() + th->BuildWorkingDir();
	Bass::SetWorkingDir(wd);
#endif

	// Write config file to working dir
	string confFileName = Bass::GetWorkingDir() + mConfig->Read<string>("command") + "-" + Logger::GetStartTimeString() + ".conf";
	mConfig->WriteFile(confFileName);

	// Configure Logger
	Logger::SetAppLogDir(Bass::GetExecDir());
	Logger::SetCommandLogDir(Bass::GetWorkingDir());
	Logger::Init(th->ShouldWriteTaskLogs());

	// Some old stuff that needs to be changed
	Bass::SetOutputLevel(mConfig->Read<uint32>("verbosity", 2));
	ECHO(1,printf(" * Verbosity:\t\t%d\n", Bass::GetOutputLevel()));
	ECHO(1, printf(" * Max Mem Usage:\t%dmb\n",(uint32)(Bass::GetMaxMemUse()/(uint64)MEGABYTE)));

	if(mConfig->KeyExists("takeiteasy") && mConfig->Read<uint32>("takeiteasy") != 0) {
		ECHO(1, printf(Bass::GetOutputLevel() > 1 ? " * Priority:\t\tYa man, i and i gonna take it easy, man...\n" : " * Priority:\t\tLow\n"))
			SystemUtils::TakeItEasy();
	} else
		ECHO(1, printf(Bass::GetOutputLevel() > 1 ? " * Priority:\t\tOpzij, opzij, opzij!\n\t\t\tMaak plaats, maak plaats, maak plaats!\n\t\t\tWij hebben ongelovelijke haast!\n" : " * Priority:\t\tHigh\n"))
	if(Bass::GetNumThreads() > 1)
		ECHO(1,printf(" * Threaded:\t\tusing %d threads\n", Bass::GetNumThreads()));

	ECHO(1,printf("\n"));

	// Execute task at hand
	th->HandleTask();	// throws error if it can't handle it, baby.

	// Finished
	ECHO(1,printf("\n * Check results in:\t%s\n", Bass::GetWorkingDir().c_str()));

	// Round up command.log
	Logger::CommandEnd();
	Logger::CleanUp();

	// Cleanup
	delete th;
}
