//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../../elobal.h"

#if defined (__unix__)
#include <glibc_s.h>
#endif

#include "BinaryColumn.h"

BinaryColumn::BinaryColumn(const uint32 id, const uint32 size, const string label) : Column(id, size, label) {
	mNoDesc = "0";
	mYesDesc = "1";

	mValues = new bool[size];
}
BinaryColumn::BinaryColumn(const uint32 id, const uint32 size, const bool *values, const string label) : Column(id, size, label) {
	mNoDesc = "0";
	mYesDesc = "1";

	mValues = new bool[size];
	memcpy(mValues, values, size * sizeof(bool));
}

BinaryColumn::BinaryColumn(BinaryColumn *t) : Column(t) {
	mNoDesc = t->mNoDesc;
	mYesDesc = t->mYesDesc;

	mValues = new bool[mSize];
	memcpy(mValues, t->mValues, mSize * sizeof(bool));
}


BinaryColumn::~BinaryColumn() {
	delete[] mValues;
}

Column* BinaryColumn::Clone() {
	return new BinaryColumn(this);
}

float BinaryColumn::SetValueFromDesc(const uint32 row, const string &desc) {
	mValues[row] = desc.compare(mYesDesc) == 0;
	return mValues[row] ? 1.0f : 0.0f;
}

void BinaryColumn::SetValueDescs(const string &no, const string &yes) {
	mNoDesc = no;
	mYesDesc = yes;
}
void BinaryColumn::GetValueDescs(string &no, string &yes) {
	no = mNoDesc;
	yes = mYesDesc;
}
void BinaryColumn::RemoveSubset(BinaryColumn *subset, const uint32 subsetSize) {
	uint32 newSize = mSize - subsetSize;
	bool *newValues = new bool[newSize];

	uint32 idx = 0;
	for(uint32 i=0; i<mSize; i++)
		if(!subset->GetBoolValue(i))
			newValues[idx++] = mValues[i];

	delete[] mValues;
	mValues = newValues;
	mSize = newSize;
}
void BinaryColumn::RemoveComplement(BinaryColumn *subset, const uint32 subsetSize) {
	uint32 newSize = subsetSize;
	bool *newValues = new bool[newSize];

	uint32 idx = 0;
	for(uint32 i=0; i<mSize; i++)
		if(subset->GetBoolValue(i))
			newValues[idx++] = mValues[i];

	delete[] mValues;
	mValues = newValues;
	mSize = newSize;
}
float BinaryColumn::ComputeDensity() {
	uint32 count = 0;
	for(uint32 i=0; i<mSize; i++)
		if(mValues[i])
			++count;
	return (float)count / mSize;
}
