//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __COLUMN_H
#define __COLUMN_H

#include <Primitives.h>

enum ColumnType {
	BinaryColumnType,
	//DoubleColumnType,
	FloatColumnType,
	NominalColumnType,
	//Uint16ColumnType,
	//Uint32ColumnType,
};

class BinaryColumn;

class Column {
public:
	Column(const uint32 id, const uint32 size, const string label);
	Column(Column *column);
	virtual ~Column();

	// -- Pure virtual --
	
	virtual Column		*Clone() =0;
	virtual ColumnType	GetType() =0;
	virtual float		SetValueFromDesc(const uint32 row, const string &desc) =0;

	virtual string		GetValueString(const uint32 r) =0;
	virtual float		GetValue(const uint32 r) =0;

	virtual void		RemoveSubset(BinaryColumn *subset, const uint32 subsetSize) =0;
	virtual void		RemoveComplement(BinaryColumn *subset, const uint32 subsetSize) =0;

	// -- Get / Set --

	void	SetId(uint32 i)				{ mId = i; }
	uint32 	GetId()						{ return mId; }

	void	SetLabel(string &l)			{ mLabel = l; }
	string	GetLabel()					{ return mLabel; }

	uint32	GetSize()					{ return mSize; }

	// -- Static --
	static Column	*Create(const ColumnType type, const uint32 id, const uint32 size, const string label);

protected:
	uint32	mId;
	uint32	mSize;
	string	mLabel;
};

#endif // __COLUMN_H
