//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../../elobal.h"

#if defined (__unix__)
#include <glibc_s.h>
#endif

#include <Bass.h>

#include "BinaryColumn.h"

#include "NominalColumn.h"

NominalColumn::NominalColumn(const uint32 id, const uint32 size, const string label) : Column(id, size, label) {
	mNumValues = 0;
	mDescToVal.clear();
	mValToDesc = NULL;

	mValues = new uint32[mSize];
}

NominalColumn::NominalColumn(NominalColumn *t) : Column(t)  {
	mNumValues = 0;
	SetValueDescs(t->mValToDesc, t->mNumValues);

	mValues = new uint32[mSize];
	memcpy(mValues, t->mValues, mSize * sizeof(uint32));
}


NominalColumn::~NominalColumn() {
	delete[] mValues;
	delete[] mValToDesc;
}

Column* NominalColumn::Clone() {
	return new NominalColumn(this);
}

float NominalColumn::SetValueFromDesc(const uint32 row, const string &desc) {
	mValues[row] = mDescToVal[desc];
	return (float)mValues[row];
}

void NominalColumn::SetValueDescs(string *valueDescs, const uint32 &numValues) {
	if(mNumValues > 0)
		THROW("Can only set value descriptions when not yet done.");
	mNumValues = numValues;
	mValToDesc = new string[mNumValues];
	for(uint32 i=0; i<mNumValues; i++) {
		mValToDesc[i] = valueDescs[i];
		mDescToVal[valueDescs[i]] = i;
	}
}
string *NominalColumn::GetValueDescs(uint32 &numDescs) {
	numDescs = mNumValues;
	return mValToDesc;
}
uint32 NominalColumn::DescToValue(const string &desc) {
	for(uint32 i=0; i<mNumValues; i++)
		if(mValToDesc[i].compare(desc)==0)
			return i;
	return UINT32_MAX_VALUE;
}
void NominalColumn::RemoveSubset(BinaryColumn *subset, const uint32 subsetSize) {
	uint32 newSize = mSize - subsetSize;
	uint32 *newValues = new uint32[newSize];

	uint32 idx = 0;
	for(uint32 i=0; i<mSize; i++)
		if(!subset->GetBoolValue(i))
			newValues[idx++] = mValues[i];

	delete[] mValues;
	mValues = newValues;
	mSize = newSize;
}
void NominalColumn::RemoveComplement(BinaryColumn *subset, const uint32 subsetSize) {
	uint32 newSize = subsetSize;
	uint32 *newValues = new uint32[newSize];

	uint32 idx = 0;
	for(uint32 i=0; i<mSize; i++)
		if(subset->GetBoolValue(i))
			newValues[idx++] = mValues[i];

	delete[] mValues;
	mValues = newValues;
	mSize = newSize;
}
