//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __NOMINALCOLUMN_H
#define __NOMINALCOLUMN_H

#include "Column.h"

typedef std::map<string,uint32> nominalDescToVal;

class NominalColumn : public Column {
public:
	NominalColumn(const uint32 id, const uint32 size, const string label = "");
	NominalColumn(NominalColumn *column);
	virtual ~NominalColumn();

	virtual Column *Clone();

	virtual ColumnType GetType() { return NominalColumnType; }

	virtual float	SetValueFromDesc(const uint32 row, const string &desc);

	virtual string	GetValueString(const uint32 r)	{ return mValToDesc[mValues[r]]; }
	virtual float	GetValue(const uint32 r)		{ return (float)mValues[r]; }

	virtual void	RemoveSubset(BinaryColumn *subset, const uint32 subsetSize);
	virtual void	RemoveComplement(BinaryColumn *subset, const uint32 subsetSize);

	// type-specific
	uint32			*GetNominalValues() { return mValues; }
	uint32			GetNumValues() { return mNumValues; }
	void			SetValueDescs(string *valueDescs, const uint32 &numValues);
	string			*GetValueDescs(uint32 &numDescs);
	string			GetValueDesc(uint32 val) { return mValToDesc[val]; }
	uint32			DescToValue(const string &desc);

protected:
	uint32				*mValues;

	uint32				mNumValues;
	nominalDescToVal	mDescToVal;
	string				*mValToDesc;
};

#endif // __NOMINALCOLUMN_H
