//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __ROW_H
#define __ROW_H

#include <Primitives.h>

class Row {
public:
	Row(const uint32 &numValues, uint32 tid = UINT32_MAX_VALUE);
	Row(Row *transaction);
	virtual ~Row();

	virtual Row *Clone();

	uint32		GetId()							{ return mId; }
	void		SetId(uint32 t)					{ mId = t; }

	float		GetValue(uint32 col)			{ return mValues[col]; }
	bool		GetBinaryValue(uint32 col)		{ return mValues[col] == 1.0; }
	uint32		GetNominalValue(uint32 col)		{ return (uint32)(mValues[col]); }

	float		*GetValues(uint32 &numValues)	{ numValues = mNumValues; return mValues; }
	uint32		GetNumValues()					{ return mNumValues; }

	void		SetValue(uint32 col, bool val)	{ mValues[col] = (val ? 1.0f : 0.0f); }
	void		SetValue(uint32 col, float val) { mValues[col] = val; }
	void		SetValue(uint32 col, uint32 val) { mValues[col] = (float)val; }

protected:
	uint32		mId; // row id

	uint32		mNumValues;
	float		*mValues;
};

#endif // __ROW_H
