//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __EMMDATASET_H
#define __EMMDATASET_H

#include "DataSet.h"

class Database;

class EMMDataSet : public DataSet {
public:
	EMMDataSet();
	EMMDataSet(EMMDataSet *dataset);
	EMMDataSet(DataSet *dataset, string searchAtts, string targetAtts);
	virtual ~EMMDataSet();

	virtual DataSet		*Clone();

	// -- Data manipulation --
	Database*	CreateDescriptionDatabase();			// A Database = binary (and originates from fic/Krimp), while
	Database*	CreateModelDatabase();
	DataSet*	CreateDescriptionDataSet();				// a DataSet can contain any type of attributes.
	DataSet*	CreateModelDataSet();

	// -- Get/Set --
	bool		HasBinaryDescriptionSpace()	{ return mBinaryDescriptionSpace; }
	bool		HasBinaryModelSpace()		{ return mBinaryModelSpace; }

	uint32		GetNumDescriptionColumns()	{ return mNumDescriptionColumns; }
	uint32		GetNumModelColumns()		{ return mNumModelColumns; }
	Column**	GetDescriptionColumns()		{ return mDescriptionColumns; }
	Column**	GetModelColumns()			{ return mModelColumns; }

	// -- Static --

	static EMMDataSet*	RetrieveDataSet(const string &dsName);
	static EMMDataSet*	ReadDataSet(const string &dsName, const string &path = "", const string &baseName = "");


protected:
	bool		mBinaryDescriptionSpace;
	bool		mBinaryModelSpace;
	uint32		mNumDescriptionColumns;
	uint32		mNumModelColumns;
	Column**	mDescriptionColumns;
	Column**	mModelColumns;
};

#endif // __EMMDATASET_H
