//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../../elobal.h"

#if defined (__unix__)
#include <glibc_s.h>
#endif

#include <Bass.h>

#include "../dataset/DataSet.h"

#include "BinarySubset.h"
#include "Uint32ArraySubset.h"

#include "DataSubset.h"

DataSubset::DataSubset(DataSet *dataSet) {
	mDataSet = dataSet;
	mSize = 0;
}
DataSubset::DataSubset(DataSubset *datasubset) {
	mDataSet = datasubset->mDataSet;
	mSize = datasubset->mSize;
}
DataSubset::~DataSubset() {

}
void DataSubset::Save(const string &filename) {
	uint32 numRows = mDataSet->GetNumRows();
	FILE *subsetFile;
	if(fopen_s(&subsetFile, filename.c_str(), "w") != 0)
		THROW("Could not open file for writing.");
	fprintf_s(subsetFile, "dss-0.1\n");
	fprintf_s(subsetFile, "sS=%d nR=%d\n", mSize, numRows); // subsetSize, numRows
	for(uint32 r=0; r<numRows; r++)
		fprintf_s(subsetFile, "%s\n", IsInSubset(r) ? "1" : "0");
	fclose(subsetFile);
}
void DataSubset::SaveLabelled(const string &filename, string* labels) {
	uint32 numRows = mDataSet->GetNumRows();
	FILE *subsetFile;
	if(fopen_s(&subsetFile, filename.c_str(), "w") != 0)
		THROW("Could not open file for writing.");
	fprintf_s(subsetFile, "Number of rows in subgroup: %d\n\nrow;label\n", mSize, numRows);
	for(uint32 r=0; r<numRows; r++)
		if(IsInSubset(r))
			fprintf_s(subsetFile, "%u;%s\n", r, labels[r].c_str());
	fclose(subsetFile);
}
DataSubset* DataSubset::Create(DataSet *ds) {
	return new BinarySubset(ds);
}
DataSubset*	DataSubset::Create(DataSet *ds, bool *mask, bool takeOwnership /*= false*/) {
	return new BinarySubset(ds, mask, takeOwnership);
}
DataSubset* DataSubset::Create(DataSubsetType type, DataSet *ds) {
	if(type == BinarySubsetType)
		return new BinarySubset(ds);
	THROW("Unknown datasubset type.");
}
DataSubset* DataSubset::Load(const string &filename, DataSet *ds) {
	FILE *subsetFile;
	if(fopen_s(&subsetFile, filename.c_str(), "r") != 0)
		THROW("Could not open file for reading.");
	uint32 minorVersion, majorVersion;
	if(fscanf_s(subsetFile, "dss-%d.%d\n", &minorVersion, &majorVersion) < 2)
		THROW("Not a recognised file format.");
	uint32 size, numRows;
	fscanf_s(subsetFile, "sS=%d nR=%d\n", &size, &numRows); // subsetSize, numRows

	if(numRows != ds->GetNumRows())
		THROW("#rows of subset and dataset do not match.");

	bool *mask = new bool[numRows];
	uint32 val;
	for(uint32 r=0; r<numRows; r++) {
		fscanf_s(subsetFile, "%d\n", &val);
		mask[r] = val == 1;
	}

	fclose(subsetFile);

	BinarySubset *dss = new BinarySubset(ds);
	dss->UseThisMask(mask, size);
	return dss;
}
