//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __DATASUBSET_H
#define __DATASUBSET_H

#include <Primitives.h>
#include <Bass.h>

enum DataSubsetType {
	BinarySubsetType = 0,
	Uint32ArraySubsetType,
	ListSubsetType,
};

class BinaryColumn;
class DataSet;

class DataSubset {
public:
	DataSubset(DataSet *ds);
	DataSubset(DataSubset *datasubset);
	virtual ~DataSubset();

	virtual DataSubset		*Clone() =0;

	// -- Interface --
	virtual void		AddToSubset(const uint32 row) =0;
	virtual bool		IsInSubset(const uint32 row) =0;
	virtual void		RemoveFromSubset(const uint32 row) =0;
	virtual bool		Equals(DataSubset *subset) =0;

	// Caller should delete array.
	virtual uint32		*GetRowsInSubset() =0;
	virtual uint32		*GetRowIdsInSubset() =0;
	virtual BinaryColumn *GetBinaryColumn() =0;
	// But not in this case:
	virtual bool		*GetMask() { THROW_NOT_IMPLEMENTED(); }
	virtual uint32		*GetArray() { THROW_NOT_IMPLEMENTED(); }

	virtual uint32		GetSize()		{ return mSize; }

	virtual void		Save(const string &filename);
	virtual void		SaveLabelled(const string &filename, string* labels);

	// -- Static --
	static DataSubset*	Create(DataSet *ds); // create default (bool mask)
	static DataSubset*	Create(DataSet *ds, bool *mask, const bool takeOwnership = false); // create default (bool mask)
	static DataSubset*	Create(DataSubsetType type, DataSet *ds);

	static DataSubset*	Load(const string &filename, DataSet *ds);

protected:
	DataSet			*mDataSet;
	uint32			mSize;
};

#endif // __DATASUBSET_H
