//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __SUBGROUP_H
#define __SUBGROUP_H

#include "../datasubset/DataSubset.h"

class ConditionArray;
class Model;

class Subgroup {
public:
	Subgroup(ConditionArray *ca, DataSubset *subset); // Subgroup takes ownership of both ConditionArray and DataSubset
	Subgroup(Subgroup *subgroup);
	virtual ~Subgroup();
	Subgroup *Clone()									{ return new Subgroup(this); }

	// The conditions
	ConditionArray	*GetConditions()					{ return mConditions; }
	void			SetConditions(ConditionArray *ca)	{ mConditions = ca; }

	// The set of rows
	void			SetSubset(DataSubset *subset)		{ mSubset = subset; mSize = subset->GetSize(); }
	DataSubset		*GetSubset()						{ return mSubset; }
	void			ForgetSubset();
	void			CacheSubset(DataSet *dataSet);
	uint32			GetSize()							{ return mSize; }
	bool			Contains(uint32 row)				{ return mSubset->IsInSubset(row); }
	bool			*GetMask()							{ return mSubset==NULL ? NULL : mSubset->GetMask(); }

	// The model
	Model			*GetModel()							{ return mModel; }
	void			SetModel(Model *model); // deletes old model

	// The quality
	double			GetQuality()						{ return mQuality; }
	void			SetQuality(double q)				{ mQuality = q; }
	double			GetOptimisticEstimate()				{ return mOptimisticEstimate; }
	void			SetOptimisticEstimate(double oe)	{ mOptimisticEstimate = oe; }

	// The rest
	uint32			GetId()								{ return mId; }
	void			SetId(uint32 id)					{ mId = id; }
	bool			IsRefined()							{ return mRefined; }
	void			SetRefined(bool r)					{ mRefined = r; }

protected:
	// Member variables
	ConditionArray	*mConditions;
	DataSubset		*mSubset;
	Model			*mModel;
	double			mQuality;
	double			mOptimisticEstimate;
	uint32			mId;
	uint32			mSize;
	bool			mRefined;	// usually not needed, only for some search types
};

#endif // __SUBGROUP_H
