//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __SUBGROUPCOLLECTION_H
#define __SUBGROUPCOLLECTION_H

class DataSet;
class Subgroup;
class QualityMeasure;

struct SubgroupCollectionStats {
	uint32	numSubgroups;
	double	qualityMin;
	double	qualityMax;
	double	qualityAvg;
	double	qualitySd;
	double	sizeAvg;
	double	sizeSd;
	double	descAvg;
	double	descSd;
	double	diversityPenalty;
	double	jointEntropy;
};

class SubgroupCollection {
public:
	SubgroupCollection(const uint32 maxSize = 0, const bool cacheSubsets = true);
	virtual ~SubgroupCollection();

	// Basic collection info
	uint32			GetMaxSize()						{ return mMaxSize; }
	uint32			GetSize()							{ return mSize; }
	uint32			GetNumCandidates()					{ return mNumCandidates; }

	double			GetMinQuality()						{ return mMinQuality; }
	double			GetMaxQuality()						{ return mMaxQuality; }

	bool			GetAutoSort()						{ return mAutoSort; }
	void			SetAutoSort(bool as)				{ mAutoSort = as; }

	// Normal operation
	bool			GoodEnough(double quality); // Check whether a subgroup with specified quality might be added to the collection

	void			Add(Subgroup *subgroup);	// Creates clone of subgroup iff subgroup is stored
	void			AddToCandidateCount(const uint32 num) { mNumCandidates += num; }
	void			CacheModels(QualityMeasure *measure);
	void			CacheSubsets(DataSet *dataSet);

	SubgroupList	*GetSubgroups()						{ return mSubgroups; }
	void			GetUnrefinedSubgroupsIn(SubgroupList *subgroups);
	void			UpdateStats();

	void			SortOnQuality();

	// Computing several scores and statistics
	double			ComputeDiversityScore(const uint32 numRows);	// Cover Redundancy; lower score means smoother coverage distribution over the rows, larger diversity
	SubgroupCollectionStats ComputeStats(const uint32 numRows);

	// Write to disk
	void			WriteStatsFile(const string &filename, DataSet *descriptionDS);
	void			WriteSubgroups(const string &filename, DataSet *descriptionDS);
	void			WriteSubsets(const string &prefix, bool outputSubsets = true, bool outputLabelledSubsets = false, string *labels = NULL);
	void			WriteModels(const string &prefix);

protected:
	void			Pushback(Subgroup *subgroup);			// no maxsize, no autosort
	void			Insert(Subgroup *subgroup);
	void			InsertAndTruncate(Subgroup *subgroup);

	// Member variables
	SubgroupList *mSubgroups;	// keep subgroup sorted descending on quality

	uint32		mMaxSize;
	uint32		mSize;
	uint32		mNumCandidates;
	bool		mAutoSort;
	bool		mCacheSubsets;

	double		mMinQuality;
	double		mMaxQuality;
};

#endif // __SUBGROUPCOLLECTION_H
