//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
// emc.cpp : Defines the entry point for the console application.

#include "elobal.h"

#include <time.h>
#if defined (_WINDOWS)
	#include <direct.h>
#elif defined (__unix__)
	#include <unistd.h>
#endif

#include <SystemUtils.h>
#include <FileUtils.h>
#include <Config.h>
#include <logger/Logger.h>

#include <Bass.h>

#include "EmcConf.h"
#include "EmcMain.h"

void ProcessConf(const string &confName, uint32 fromInst=0, uint32 toInst=0);
void ProcessBatch(const string &batchName);

#define VERSION_MARK 0
#define VERSION_REVISION 1

int main(int argc, char* argv[])
{
#ifdef _MEMLEAK
	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
	_CrtSetReportMode ( _CRT_ERROR, _CRTDBG_MODE_DEBUG);
#endif
	// break on allocation of specific memory block
	 //_crtBreakAlloc = 32298;

	string arg;
	bool pressEnter = false;

	const string defaultConf = "emc.conf";
	{
		arg = argv[0];
		string::size_type loc = arg.find_last_of("\\/");
		if(loc != string::npos) {
			string ed = arg.substr(0, loc+1);
			Bass::SetExecDir(ed);
		} else {
#if defined (_MSC_VER)
			Bass::SetExecDir(FileUtils::GetCurDirectory());
#elif defined (__GNUC__)
			string ed(FileUtils::GetCurDirectory());
			Bass::SetExecDir(ed);
#endif
		}
		Bass::SetExecName(arg);
	}

	uint32 f,t;
	switch(argc) {
		case 1:
			if(arg.compare("-version") == 0) {
				printf("EMC mark %s revision %s", VERSION_MARK, VERSION_REVISION);
			} else {
				ProcessConf(defaultConf);
			}
			break;
		case 2:
			arg = argv[1];
			if(arg.find("\\") != string::npos || arg.find("/") != string::npos) {
				string::size_type loc = arg.find_last_of("\\/");
				string cwd = arg.substr(0, loc);
#if defined (_WINDOWS)
				_chdir(cwd.c_str()); /* check success ? */
#elif defined (__unix__)
				chdir(cwd.c_str()); /* check success ? */
#endif
				printf("Changed working directory to: %s\n", cwd.c_str());
				pressEnter = true;
			}
			if(arg.find(".batch") != string::npos)
				ProcessBatch(arg);
			else
				ProcessConf(arg);
			break;
		case 3:
			arg = argv[1];
			if(arg.compare("batch") == 0) {
				arg = argv[2];
				ProcessBatch(arg);
			} else if(/*arg.find(".conf") != string::npos &&(*/ sscanf(argv[2],"%d",&f)) {
				ProcessConf(argv[1],f);				
			} else 
				printf("Unknown command: %s\n\n", argv[1]);
			break;
		case 4:
			arg = argv[1];
			if(arg.find(".conf") != string::npos && sscanf(argv[2],"%d",&f) && sscanf(argv[3],"%d",&t)) {
				ProcessConf(argv[1],f,t);
			} else 
				printf("Unknown command: %s\n\n", argv[1]);
			break;
		default:
			printf("Error. Wrong number of arguments.\n\n");
	}
	if(pressEnter) {
		printf("\n\nPress enter ...");
		getchar();
	} 
	return 0;
}

void ProcessBatch(const string &batchName) {
	FILE *fp = NULL;
	char *buffer = new char[100];
	fp = fopen(batchName.c_str(), "r");
	string confName;

	if(fp != NULL) {
		while(fgets(buffer, 100, fp)) {
			confName = buffer;
			confName.resize(confName.size() - 1);
			if(confName.size() > 1 && confName.at(0) != '#')
				ProcessConf(confName);
			else
				printf("Batch -- skipping '%s'\n", confName.c_str());
		}
		if(ferror(fp))
			printf("* Error while reading batch file.\n");
		fclose(fp);

	} else {	// File open error!
		printf("* Could not read batch file.\n");
	}
	delete[] buffer;
}

void ProcessConf(const string &confName, uint32 fromInst, uint32 toInst) {
	printf(" ---------------------------------- DSSD release 1 ---------------------------------\n\n");
	printf("** Processing conf: '%s'\n", confName.c_str());

	try { 
		EmcConf emcConf;
		emcConf.Read(confName);

		uint32 numInsts = emcConf.GetNumInstances();
		if((fromInst > 0 && fromInst > numInsts) || (toInst > 0 && toInst > numInsts))
			throw "Ho! ho! ho! There arent that many instances in the config file";

		if(numInsts > 1 && ((fromInst == 0 && toInst == 0) || (fromInst > 0 && toInst > 0 && fromInst < toInst) )) {
			if(fromInst > 0) fromInst--;
			if(toInst > 0) toInst--;
			else if(toInst == 0) toInst = numInsts-1;

			for(uint32 curInst=fromInst; curInst <= toInst; curInst++) {
				Config* instConf = emcConf.GetConfig(curInst);
				EmcMain main(instConf);
				try {
					main.Execute();
					delete instConf;
				} catch(...) {
					delete instConf;
					throw;
				}
				printf("\n\n");
#if defined (_WINDOWS)
				Sleep(1000);
#elif defined (__unix__)
				sleep(1);
#endif
			}
		} else {
			if(fromInst > 0) fromInst--;
			Config* instConf = emcConf.GetConfig(fromInst);
			EmcMain main(instConf);
			try {
				main.Execute();
				delete instConf;
			} catch(string ex) {
				Logger::CleanUp();
				delete instConf;
				throw ex;
			} catch(...) {
				Logger::CleanUp();
				delete instConf;
				throw;
			}
		}
	} catch(char *msg) {
		printf("\n!! Run-time fatal exception:\n ! %s\n", msg);
	} catch(string msg) {
		printf("\n!! Run-time fatal exception:\n ! %s\n", msg.c_str());
	} catch(Config::key_not_found knf) {
		printf("\n\nFatal: Required key '%s' not found in config file.", knf.key.c_str());
	}
}
