//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../elobal.h"

#include <Bass.h>
#include <Config.h>
#include <logger/Log.h>

#include "../datastructs/dataset/DataSet.h"
#include "../datastructs/datasubset/DataSubset.h"
#include "../datastructs/data/Column.h"
#include "../datastructs/subgroup/Subgroup.h"

#include "../models/ContingencyTable.h"

#include "ChiSquaredMeasure.h"

ChiSquaredMeasure::ChiSquaredMeasure(Config *config, DataSet *ds) : QualityMeasure(config, ds) {
	// Configuration

	// -- Build model for overall dataset --
	if(ds->GetNumColumns() > 1)
		THROW("Currently no support for multiple columns.");
	mDSModel = ContingencyTable::Build(ds);
	ds->SetModel(mDSModel);
}
ChiSquaredMeasure::~ChiSquaredMeasure() {
	// mDSModel is taken over by DS
}
void ChiSquaredMeasure::LogParameters() {
	QualityMeasure::LogParameters();
}

double ChiSquaredMeasure::ComputeQuality(Subgroup *subgroup, const bool computeOE /*= false*/) {
	// Get us the subset
	DataSubset *subset = subgroup->GetSubset();

	// Build ContingencyTable
	ContingencyTable *sgModel = ContingencyTable::Build(mDS, subset);

	// Init some variables
	double quality = 0.0;
	uint32 numValues = sgModel->GetNumValuesForColumn(0);
	uint32 *sgCounts = sgModel->GetCountsForColumn(0);
	uint32 *dbCounts = mDSModel->GetCountsForColumn(0);
	uint32 sgSize = subset->GetSize();
	double divider = (double)mDSNumRows;

	// Compute chi-squared
	double someValue;
	for(uint32 i=0; i<numValues; i++) {
		if(dbCounts[i]>0) {
			someValue = (dbCounts[i] * sgSize) / divider;
			quality += pow(sgCounts[i] - someValue, 2.0) / someValue;
			someValue = (dbCounts[i] * (mDSNumRows - sgSize)) / divider;
			quality += pow((dbCounts[i] - sgCounts[i]) - someValue, 2.0) / someValue;
		}
	}

	// Compute OE if asked for
	/*if(computeOE) {
		double oe;

		subgroup->SetOptimisticEstimate(oe);
	}*/

	// Save to subgroup
	SaveToSubgroup(subgroup, sgModel, quality);

	// Return quality
	return quality;
}
void ChiSquaredMeasure::CacheModel(Subgroup *subgroup) {
	subgroup->SetModel(ContingencyTable::Build(mDS, subgroup->GetSubset()));
}
