//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../elobal.h"

#include <Bass.h>
#include <Config.h>
#include <logger/Log.h>

#include "../datastructs/dataset/DataSet.h"
#include "../datastructs/datasubset/DataSubset.h"
#include "../datastructs/subgroup/Subgroup.h"
#include "../datastructs/data/FloatColumn.h"

#include "../models/NumericArray.h"

#include "WRAccNumeric.h"

WRAccNumeric::WRAccNumeric(Config *config, DataSet *ds) : QualityMeasure(config, ds) {
	if(!ds->AllColumnsNumeric())
		THROW("Only numeric model columns supported.");

	// Configuration

	// -- Build model for overall dataset --
	mDSModel = NumericArray::BuildMean(ds);
	ds->SetModel(mDSModel);
	mDSMean = mDSModel->GetValue(0);
	mDSSize = ds->GetNumRows();
}
WRAccNumeric::~WRAccNumeric() {
	// mDSModel is taken over by DS
}
void WRAccNumeric::LogParameters() {
	QualityMeasure::LogParameters();
}

double WRAccNumeric::ComputeQuality(Subgroup *subgroup, const bool computeOE /*= false*/) {
	// Init
	DataSubset *subset = subgroup->GetSubset();

	// Build ContingencyTable
	NumericArray *model = NumericArray::BuildMean(mDS, subset);

	// Compute WRAcc
	double quality = (subset->GetSize() / mDSSize) * (model->GetValue(0) - mDSMean);

	// Save to subgroup
	SaveToSubgroup(subgroup, model, quality);

	// Return quality
	return quality;
}
void WRAccNumeric::CacheModel(Subgroup *subgroup) {
	subgroup->SetModel(NumericArray::BuildMean(mDS, subgroup->GetSubset()));
}
