//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../../elobal.h"

#include <Bass.h>
#include <Config.h>
#include <logger/Log.h>

#include "../../datastructs/dataset/DataSet.h"
#include "../../datastructs/datasubset/DataSubset.h"
#include "../../datastructs/data/BinaryColumn.h"
#include "../../datastructs/data/FloatColumn.h"
#include "../../datastructs/data/NominalColumn.h"
#include "../../datastructs/subgroup/Subgroup.h"

#include "../../models/compression/KLCompressor.h"
#include "../../models/ContingencyTable.h"

#include "KLQualityMeasure.h"

KLQualityMeasure::KLQualityMeasure(Config *config, DataSet *ds) : CompressionBasedMeasure(config, ds) {
	// Configuration
	char firstChar = config->Read<string>("measure").at(0);
	mWeightedVersion =  firstChar == 'W' || firstChar == 'w';

	// -- Build model for overall dataset --
	mDSModel = new KLCompressor(ds);
	ds->SetModel(mDSModel);

	// Compute encoded size of DS
	mDSEncodedSize = mDSModel->GetEncodedSize();
}
KLQualityMeasure::~KLQualityMeasure() {
	// mDSModel is taken over by DS
}

double KLQualityMeasure::ComputeQuality(Subgroup *subgroup, const bool computeOE /*= false*/) {
	// Init
	DataSubset *subset = subgroup->GetSubset();

	// Build ContingencyTable
	KLCompressor *model = new KLCompressor(mDS, subset);

	// Compute total KL divergence
	double quality = mDSModel->ComputeKLDivergence(model);
		
	// Weight by size if asked for
	if(mWeightedVersion) // !weighted = bits per transactions on average; weighted = total number of bits for the subgroup
		quality *= subset->GetSize();

	// Save to subgroup
	SaveToSubgroup(subgroup, model, quality);

	// Return quality
	return quality;
}
void KLQualityMeasure::CacheModel(Subgroup *subgroup) {
	subgroup->SetModel(new KLCompressor(mDS, subgroup->GetSubset()));
}
