//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __CONTINGENCYTABLE_H
#define __CONTINGENCYTABLE_H

class DataSet;
class DataSubset;

#include "Model.h"

class ContingencyTable : public Model {
public:
	ContingencyTable(const uint32 numCols, uint32 *numValues, uint32 **counts, const uint32 numRows);
	ContingencyTable(ContingencyTable *ctable);
	ContingencyTable();
	virtual ~ContingencyTable();

	virtual Model	*Clone();

	uint32			GetNumColumns()									{ return mNumColumns; }
	uint32			GetNumValuesForColumn(const uint32 c)			{ return mNumValues[c]; }
	uint32			*GetCountsForColumn(const uint32 c)				{ return mCounts[c]; }
	float			*GetSharesForColumn(const uint32 c)				{ return mShares[c]; }
	uint32			GetCount(const uint32 c, const uint32 v)		{ return mCounts[c][v]; }
	float			GetShare(const uint32 c, const uint32 v)		{ return mShares[c][v]; }

	// Write to disk
	virtual void	WriteHumanReadable(const string &filename);

	// -- Static --
	static ContingencyTable *Build(DataSet *ds);
	static ContingencyTable *Build(DataSet *ds, DataSubset *subset);

protected:
	// Member variables
	uint32		mNumColumns;
	uint32		*mNumValues;
	uint32		**mCounts;
	float		**mShares;
};

#endif // __CONTINGENCYTABLE_H
