//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../elobal.h"

#include <Bass.h>
#include <Config.h>
#include <logger/Log.h>

#include "../datastructs/data/FloatColumn.h"
#include "../datastructs/dataset/DataSet.h"
#include "../datastructs/datasubset/DataSubset.h"

#include "NumericArray.h"

NumericArray::NumericArray(const uint32 numVals, double *values) {
	mNumValues = numVals;
	mValues = values;
}
NumericArray::NumericArray(NumericArray *narray) {
	mNumValues = narray->mNumValues;
	mValues = new double[mNumValues];
	memcpy(mValues, narray->mValues, mNumValues * sizeof(double));
}
NumericArray::NumericArray() {
	mNumValues = 0;
	mValues = NULL;
}
NumericArray::~NumericArray() {
	if(mNumValues > 0) {
		delete[] mValues;
	}
}
Model *NumericArray::Clone() {
	return new NumericArray(this);
}

void NumericArray::WriteHumanReadable(const string &filename) {
	// Open file
	string fullPath = filename + ".csv";
	FILE *fp;
	fopen_s(&fp, fullPath.c_str(), "w");

	// For each value
	for(uint32 i=0; i<mNumValues; i++) {
		fprintf_s(fp, "%f;", mValues[i]);
	}
	fseek(fp, -1, SEEK_CUR);
	fprintf_s(fp, "\n");

	// Close file
	fclose(fp);
}
NumericArray *NumericArray::BuildMean(DataSet *ds) {
	uint32 numColumns = ds->GetNumColumns();
	Column **columns = ds->GetColumns();
	uint32 numRows = ds->GetNumRows();
	double *means = new double[numColumns];

	for(uint32 i=0; i<numColumns; i++) {
		FloatColumn *col = (FloatColumn *)(columns[i]);
		float *values = col->GetFloatValues();
		double sum = 0.0;

		for(uint32 j=0; j<numRows; j++)
			sum += values[j];

		means[i] = sum / numRows;
	}

	return new NumericArray(numColumns, means);
}
NumericArray *NumericArray::BuildMean(DataSet *ds, DataSubset *subset) {
	uint32 numColumns = ds->GetNumColumns();
	Column **columns = ds->GetColumns();
	uint32 numRows = subset->GetSize();
	uint32 *rows = subset->GetRowsInSubset();
	double *means = new double[numColumns];

	for(uint32 i=0; i<numColumns; i++) {
		FloatColumn *col = (FloatColumn *)(columns[i]);
		float *values = col->GetFloatValues();
		double sum = 0.0;

		for(uint32 j=0; j<numRows; j++)
			sum += values[rows[j]];

		means[i] = sum / numRows;
	}

	delete[] rows;

	return new NumericArray(numColumns, means);
}
