//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../elobal.h"

#include <Bass.h>
#include <Config.h>
#include <StringUtils.h>
#include <logger/Log.h>

#include "DominancePruner.h"
#include "EqualCoverEliminator.h"
#include "QualitySorter.h"

#include "PostProcessor.h"

PostProcessor::PostProcessor() {

}
PostProcessor::~PostProcessor() {

}
// Static
void PostProcessor::Process(string &proc, SubgroupCollection *collection, DataSet *ds, QualityMeasure *measure) {
	// Parse processors
	uint32 numProcs;
	string *procs = StringUtils::Tokenize(proc, numProcs, "-");

	// For each processor
	for(uint32 i=0; i<numProcs; i++) {
		PostProcessor *pp = PostProcessor::Create(procs[i]);
		LOG_MSG("Applying post-processor:\t" + string(procs[i]));
		pp->Process(collection, ds, measure);
		delete pp;
	}

	// Cleanup
	delete[] procs;
}
PostProcessor* PostProcessor::Create(const string &desc) {
	if(desc.compare("dominance") == 0)
		return new DominancePruner();
	if(desc.compare("equalcover") == 0)
		return new EqualCoverEliminator();
	if(desc.compare("qualitysort") == 0)
		return new QualitySorter();
	THROW("Unknown post-processor type.");
}
