//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __ITERATIVESEARCH_H
#define __ITERATIVESEARCH_H

#include "Search.h"

// Iterative search
// E.g. sequential or weighted covering

class IterativeSearch : public Search {
public:
	IterativeSearch(const float weight, const string &searchDesc, Config *config, DataSet *descDS, DataSet *modelDS);
	virtual ~IterativeSearch();

	// -- Virtual --
	virtual SubgroupCollection	*DoSearch();
	virtual void				LogParameters();
	virtual string				GetName();
	virtual bool				CanUseOptimisticEstimate()		{ return mSearch->CanUseOptimisticEstimate(); }

protected:
	void			SequentialCover(Subgroup *subgroup);
	void			WeightedCover(Subgroup *subgroup);

	float			mWeight; // use 0.0 for sequential covering, (0.0,1.0) for multiplicative weighted covering
	string			mSearchDesc;
	Config			*mSearchConfig;
	Search			*mSearch; // not for using, only for getting some information..

	DataSet			*mUpdatedDescDataSet;
	DataSet			*mUpdatedModelDataSet;

	// Does this instance of search allow for intermediate searching?
	virtual	bool	HasIntermediateSave() { return true; }
};

#endif // __SEARCH_H
