//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __REFINEMENTOPERATOR_H
#define __REFINEMENTOPERATOR_H

#include <Primitives.h>
#include <Bass.h>

class Column;
class BinaryColumn;
class FloatColumn;
class NominalColumn;
class Config;
class ConditionArray;
class CandidateQueue;
class DataSet;
class Subgroup;

class RefinementOperator {
public:
	RefinementOperator(Config *config, DataSet *ds);
	virtual ~RefinementOperator();

	virtual void Refine(Subgroup *subgroup, CandidateQueue *candQueue);
	virtual void RefineOrdered(Subgroup *subgroup, CandidateQueue *candQueue);

	// -- Static --
	static RefinementOperator *Create(Config *config, DataSet *ds);

protected:
	// These methods check whether a possible refinement makes sense given the current conditions.
	void RefineColumn(Column *column, Subgroup *subgroup, CandidateQueue *candQueue);
	void RefineBinary(BinaryColumn *column, Subgroup *subgroup, CandidateQueue *candQueue);
	void RefineFloat(FloatColumn *column, Subgroup *subgroup, CandidateQueue *candQueue);
	void RefineNominal(NominalColumn *column, Subgroup *subgroup, CandidateQueue *candQueue);

	// These methods simply add all possible refinements, do no checks whatsoever (for possible redundant conditions); used by RefineOrdered, for exhaustive search.
	void RefineColumnNoChecks(Column *column, Subgroup *subgroup, CandidateQueue *candQueue);
	void RefineBinaryNoChecks(BinaryColumn *column, Subgroup *subgroup, CandidateQueue *candQueue);
	void RefineFloatNoChecks(FloatColumn *column, Subgroup *subgroup, CandidateQueue *candQueue);
	void RefineNominalNoChecks(NominalColumn *column, Subgroup *subgroup, CandidateQueue *candQueue);

	DataSet		*mDataSet;

	Column		**mColumns;
	uint32		mNumCols;
	uint32		*mColIds;

	uint32		mFloatNumSplits;
	float		*mTempFloats;
};

#endif // __REFINEMENTOPERATOR_H
