//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __SEARCH_H
#define __SEARCH_H

#include <Primitives.h>
#include <Bass.h>

class Config;
class DataSet;
class SubgroupCollection;
class RefinementOperator;
class QualityMeasure;
class Timer;

class Search {
public:
	Search(Config *config, DataSet *descDS, DataSet *modelDS);
	virtual ~Search();

	// -- Virtual --
	virtual SubgroupCollection	*DoSearch() =0;
	virtual void				LogParameters();
	virtual string				GetName() =0;
	virtual bool				CanUseOptimisticEstimate()		{ return false; }

	// -- Get/set --
	QualityMeasure				*GetQualityMeasure()			{ return mMeasure; }
	uint32						GetElapsedTime(); // in minutes

	// -- Static --
	static Search* Create(Config *config, DataSet *descDS, DataSet *modelDS);
	static Search* Create(const string &desc, Config *config, DataSet *descDS, DataSet *modelDS);

protected:
	// Does this instance of search allow for intermediate searching?
	virtual	bool	HasIntermediateSave() { return false; }
	// Perform this save
	void			DoIntermediateSave(const uint32 depth, SubgroupCollection *results);

	void			InitTimer();		// starts timer
	bool			OutOfTime();

	// The important variables
	Config		*mConfig;
	DataSet		*mDescDataSet;
	DataSet		*mModelDataSet;

	RefinementOperator		*mRefineOp;
	QualityMeasure			*mMeasure;

	// Generic search parameters
	uint32		mMinCoverage;
	uint32		mMaxDepth;
	bool		mDisableOptimisticEstimate;
	bool		mUseOptimisticEstimate;

	bool		mIntermediateSave;
	string		mIntermediateSaveDir;
	Timer		*mTimer;
	time_t		mMaxTime;
};

#endif // __SEARCH_H
