//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#ifndef __BEAMSELECTOR_H
#define __BEAMSELECTOR_H

class Config;
class DataSet;

class BeamSelector {
public:
	BeamSelector(Config *config, DataSet *descDS, DataSet *modelDS);
	virtual ~BeamSelector();

	// -- Virtual --
	virtual void		Select(SubgroupList *subgroups, SubgroupList *beam, const uint32 depth, const bool takeOwnership) =0; // if takeOwnership: foreach sg in subgroup: either use for beam or delete!

	virtual void		LogParameters();
	virtual string		GetConfigString();
	virtual string		GetName() =0;

	// -- Static --
	static BeamSelector* Create(Config *config, DataSet *descDS, DataSet *modelDS);

protected:
	void				AddToBeam(SubgroupList *beam, Subgroup *subgroup, const bool takeOwnership);

	uint32				mMaxBeamWidth;
	float				mVariableParam;
	bool				mUseVariableWidth;

	DataSet				*mDescDS;
	DataSet				*mModelDS;
	uint32				mNumRows; // not really required here, but often needed so for the sake of convenience..
};

#endif // __BEAMSELECTOR_H
