//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../../elobal.h"

#if defined (__unix__)
#include <glibc_s.h>
#endif

#include <Config.h>
#include <logger/Log.h>

#include "../../datastructs/subgroup/Subgroup.h"

#include "QualityBeam.h"

QualityBeam::QualityBeam(Config *config, DataSet *descDS, DataSet *modelDS) : BeamSelector(config, descDS, modelDS) {

}
QualityBeam::~QualityBeam() {

}
void QualityBeam::LogParameters() {
	BeamSelector::LogParameters();
}
void QualityBeam::Select(SubgroupList *subgroups, SubgroupList *beam, const uint32 depth, const bool takeOwnership) {
	SubgroupList::iterator it = subgroups->begin();

	// Always select the highest ranking subgroup
	AddToBeam(beam, *it, takeOwnership);
	double minQuality = mUseVariableWidth ? mVariableParam*(*it)->GetQuality() : FLOAT_MIN_VALUE;

	// Skip to next
	it++;

	for(uint32 numSelected=1; numSelected<mMaxBeamWidth && it!=subgroups->end(); ++it, ++numSelected) { // assume that subgroups are already sorted descending on quality
		if((*it)->GetQuality() < minQuality)
			break;
		AddToBeam(beam, *it, takeOwnership);
	}

	// Delete all items that have not been selected for the beam
	if(takeOwnership)
		for(; it!=subgroups->end(); ++it)
			delete *it; // cleanup unused subgroups immediately
}
