//
// Copyright (c) 2011-2012, Matthijs van Leeuwen
//
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
// 
// Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
// Neither the name of the Universiteit Utrecht, KU Leuven, nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include "../elobal.h"

#if defined (__unix__)
#include <glibc_s.h>
#endif

#include <ArrayUtils.h>
#include <Config.h>
#include <logger/Log.h>

#include "../../datastructs/subgroup/Subgroup.h"

#include "SlowDescendBeam.h"

SlowDescendBeam::SlowDescendBeam(Config *config, DataSet *modelDS) : FixedSizeBeam(config, modelDS) {
	mMaxDepth		= config->Read<uint32>("maxDepth"); // not really elegant this way.. should be taken from Search (somehow)
}
SlowDescendBeam::~SlowDescendBeam() {

}
void SlowDescendBeam::LogParameters() {
	FixedSizeBeam::LogParameters();
}
void SlowDescendBeam::Select(SubgroupList *subgroups, SubgroupList *beam, const uint32 depth, const bool takeOwnership) {
	// Init
	uint32 numSubgroups = subgroups->size();
	double *correctedScores = new double[numSubgroups];
	uint32 *idxArray = new uint32[numSubgroups];
	Subgroup **sgArray = new Subgroup *[numSubgroups];
	double numRows = (double)mNumRows;

	// Get subgroups and compute scores
	uint32 idx = 0;
	for(SubgroupList::iterator it=subgroups->begin(); it!=subgroups->end(); ++it) {
		sgArray[idx] = *it;
		idxArray[idx] = idx;
		correctedScores[idx++] = (*it)->GetQuality() * pow((*it)->GetSize() / numRows, ((double)mMaxDepth - depth)/(mMaxDepth - 1)); // q * relSize^((dmax - d)/(dmax - 1))
	}

	// Sort on corrected scores
	ArrayUtils::MSortArrayDesc<double>(correctedScores, numSubgroups, idxArray);

	// Fill beam
	uint32 numSelect = min(mBeamSize, numSubgroups);
	for(uint32 i=0; i<numSelect; i++)
		AddToBeam(beam, sgArray[idxArray[i]], takeOwnership);

	// Cleanup the rest if ownership should be taken
	if(takeOwnership) {
		for(uint32 i=numSelect; i<numSubgroups; i++)
			delete sgArray[idxArray[i]];
		subgroups->clear();
	}

	// Cleanup temp structs
	delete[] correctedScores;
	delete[] idxArray;
	delete[] sgArray;
}
